/*
	============================================================================
	Name        : preesm.h
	Author      : kdesnos
	Version     : 1.0
	Copyright   : CECILL-C, IETR, INSA Rennes
	Description : Usefull declarations for all headers used in preesm.
	============================================================================
*/

#ifndef PREESM_H
#define PREESM_H

#define _PREESM_PAPIFY_MONITOR
#define PREESM_NO_AFFINITY

#define IN
#define OUT
//#define _PREESM_PAPIFY_MONITOR
//#define _PREESM_MONITOR_INIT
//#ifdef _PREESM_MONITOR_INIT
//#include "eventLib.h"
//#endif



/**
 *
 * gettimeofday(&t0, NULL);
 * ....
 * gettimeofday(&tf, NULL);
 * t_hw = ((tf.tv_sec - t0.tv_sec) * 1000.0) + ((tf.tv_usec - t0.tv_usec) / 1000.0);
 * printf("Total execution time: %.6f ms\n", t_hw);
 *
 *
 */
#define MEASURE_TIME
#ifdef MEASURE_TIME
#include <sys/time.h> // struct timeval, gettimeofday()
#include <time.h>     // time()
struct timeval t0, tf;
float t_hw, t_sw;
#endif

#undef NDEBUG //for enabling assertions

#define PREESM_LOOP_SIZE (110000)

#define VERBOSE_ARTICO3_TIME_MEASUREMENT

#define RHO      1 //reflection coef
#define CHI      2 //expantion coef
#define GAMMA    0.5 //contraction coef
#define SIGMA    0.5 //reduction coef

// define the point in the simplex (x) and its functional value (fx)
typedef struct{
    double x[5]; // angles			// 8 * 5 = 40 Bytes
    double fx; // function cost		// 8 Bytes
    double coordinates[3]; //coordinates in the 3D space  // 8 * 3 = 24 Bytes
    double vertexCondX; 			// 8 Bytes
	double vertexCondF; 			// 8 Bytes
} point_t; // 88 Bytes


// for this application, the simplex is a set of 6 vertices (nbParameters + 1) = (5+1) = 6
typedef struct{
    point_t vertex[6];  // (8 * 11) * 6 = 88 * 6 = 528 Bytes
    double simplexCondX; // 8 Bytes
	double simplexCondF; // 8 Bytes
} simplex_t; // 544 Bytes

//
typedef struct{
    point_t Ar; 		// 8 * 11 = 88 Bytes
    point_t Ae; 		// 8 * 11 = 88 Bytes
    point_t AcOutside; 	// 8 * 11 = 88 Bytes
    point_t AcInside;  	// 8 * 11 = 88 Bytes
    simplex_t ShrinkedSimplex; // 544 Bytes
} simplexPlus_t; // 896 Byte

#define MAX_PARALLEL_POINTS (64)
extern double copyAngles[5];
extern unsigned int readCommand;
extern unsigned int genCommand;
extern unsigned long long mask_TOL_;
extern unsigned long long mask_TOL_reference;

#endif // PREESM_H
