#include "auxiliars.h"
#include <string.h>

extern unsigned int segmentCounter;

#ifdef VERBOSE_ARTICO3_TIME_MEASUREMENT

void printSimplex(int nbParameters, point_t *simplex){
	for(int i = 0; i < (nbParameters +1); i++){
		//variable
		for(int j = 0; j < nbParameters; j++){
			printf("\t%f",simplex[i].x[j]);
		}
		//coordinates
		printf("\t---> ");
		for(int k = 0; k < 3; k++){
			printf("\t%f",simplex[i].coordinates[k]);
		}
		//fx
		//printf("\t---> %f",simplex[i].fx);
		printf("\n");
	}
	printf("\n\n");
}

void printSimplex72(int nbParameters, simplex_t *simplex){
	for(int i = 0; i < (nbParameters +1); i++){
		//variable
		for(int j = 0; j < nbParameters; j++){
			printf("; %f",simplex->vertex[i].x[j]);
		}
		//coordinates
		printf("\t---> ");
		for(int k = 0; k < 3; k++){
			printf("; %f",simplex->vertex[i].coordinates[k]);
		}
		//fx
		printf("---> %f",simplex->vertex[i].fx);
		//condf and condx
		printf(" ---> %f; %f;", simplex->vertex[i].vertexCondF, simplex->vertex[i].vertexCondX);
		printf("\n");
	}
	printf("\n\n");
}


int iterationSimplexPrinter = 0;


void printSimplexIteration(int nbParameters, point_t *simplex){
	printf("SIMPLEX %d\n",iterationSimplexPrinter++);
	for(int i = 0; i < (nbParameters +1); i++){
		//variable
		for(int j = 0; j < nbParameters; j++){
			printf("\t%f",simplex[i].x[j]);
		}
		//coordinates
		//fx
		printf("\t---> %f",simplex[i].fx);
		printf("\n");
	}
	printf("\n\n");
}

void printVertex(int nbParameters, point_t *vertex){
	for(int i = 0; i < (0 +1); i++){
			//variable
			for(int j = 0; j < nbParameters; j++){
				printf(" %f, ",vertex[i].x[j]);
			}
			printf(" --> ");
			//coordinates
			for(int k = 0; k < 3; k++){
				printf(" %f, ", vertex[i].coordinates[k]);
			}
			//fx
			printf(" ---> %f",vertex[i].fx);
			printf("\n");
		}
		//printf("\n\n");
}

void printSimplexPlus(int nbParameters, simplexPlus_t *SimplexPlus){
	//
	printVertex(nbParameters, &(SimplexPlus->Ar ) );
	printVertex(nbParameters, &(SimplexPlus->Ae ) );
	printVertex(nbParameters, &(SimplexPlus->AcOutside) );
	printVertex(nbParameters, &(SimplexPlus->AcInside) );
	//TODO shrinked simplex
	printVertex(nbParameters, &(SimplexPlus->ShrinkedSimplex.vertex[0]) );
	printVertex(nbParameters, &(SimplexPlus->ShrinkedSimplex.vertex[1]) );
	printVertex(nbParameters, &(SimplexPlus->ShrinkedSimplex.vertex[2]) );
	printVertex(nbParameters, &(SimplexPlus->ShrinkedSimplex.vertex[3]) );
	printVertex(nbParameters, &(SimplexPlus->ShrinkedSimplex.vertex[4]) );
	printVertex(nbParameters, &(SimplexPlus->ShrinkedSimplex.vertex[5]) );
	printf("\n\n");
}

#else //#ifdef VERBOSE_ARTICO3_TIME_MEASUREMENT
void printSimplex(int nbParameters, point_t *simplex){
	return;
}
void printSimplexIteration(int nbParameters, point_t *simplex){
	return;
}
void printVertex(int nbParameters, point_t *vertex){
	return;
}
#endif //#ifdef VERBOSE_ARTICO3_TIME_MEASUREMENT

void replacePoint(int nbParameters, point_t *srcPoint, point_t *destPoint){
	//iterating on the nbParameters
	for(int i = 0; i < nbParameters; i++){
		destPoint[0].x[i] = srcPoint[0].x[i];
		//printf("\t%f",newPoint[0].x[i]);
	}
	//printf("\n");
	//iterating on the target (really necessary? YES)
	for(int i = 0; i < 3; i++){
		destPoint[0].coordinates[i] = srcPoint[0].coordinates[i];
	}
	//copying the value of the costFunction of this new vertex of the simplex
	destPoint[0].fx = srcPoint[0].fx;
}

//new is the destination and old is the source
void updateSimplex(int nbParameters, point_t *newSimplex, point_t *oldSimplex){
	// iterating to all vertex
	for(int i =0; i < (nbParameters + 1); i++)
		replacePoint(nbParameters, &oldSimplex[i], &newSimplex[i]);
}

void updateSimplex72(int nbParameters, simplex_t *newSimplex, simplex_t *oldSimplex){
	// iterating to all vertex
	memcpy(newSimplex,oldSimplex,sizeof(simplex_t));
}

void copyTarget(int nbParameters,point_t *sourcePoint, point_t *destPoint){
	for(int i = 0; i < 3; i++)
		destPoint[0].coordinates[i] = sourcePoint[0].coordinates[i];
}

double modulus(double x) {
    return (x>0)? x : -x;
}

void calculateCONDX(int nbParameters, simplex_t *inputSimplex){
	//
	double temp = 0;
	inputSimplex->vertex[0].vertexCondX = 0.0;
	inputSimplex->simplexCondX = 0.0;
	for(int i=1; i< nbParameters+1; i++) {
	    for(int j=0; j<nbParameters; j++) {
	    	temp = modulus(inputSimplex->vertex[0].x[j]-inputSimplex->vertex[i].x[j]);
	    	if(inputSimplex->vertex[i].vertexCondX < temp) {
	    		inputSimplex->vertex[i].vertexCondX = temp;
	    	}
	    }
	    if (inputSimplex->vertex[i].vertexCondX > inputSimplex->simplexCondX){
	    	inputSimplex->simplexCondX = inputSimplex->vertex[i].vertexCondX;
	    }
	}
	return;
}

//condf of every vertex of the simplex
void calculateCONDF(int nbParameters, simplex_t *inputSimplex){
	//
	//double tempCONDF = 0.0; // the minumum value of the mathematical function "modulus"
	(inputSimplex)->simplexCondF = 0.0;
	inputSimplex->vertex[0].vertexCondF = 0;
	for(int i=1; i<nbParameters+1; i++) {
		inputSimplex->vertex[i].vertexCondF = modulus(inputSimplex->vertex[0].fx-inputSimplex->vertex[i].fx);
		if (inputSimplex->vertex[i].vertexCondF > (inputSimplex)->simplexCondF){
			(inputSimplex)->simplexCondF = inputSimplex->vertex[i].vertexCondF;
		}
	}
	return;
}

void PRINTING_MASK_BIT(int ParallelPoints){
//	for(int i = ParallelPoints -1; i >= 0; i--){
	for(int i = 16; i >= 0; i--){
		printf("  %llu",BitVal(mask_TOL_,i));
	}
	printf("\n");
}

//mask_TOL_reference
void PRINTING_mask_TOL_reference(int ParallelPoints){
//	for(int i = ParallelPoints -1; i >= 0; i--){
	for(int i = 16; i >= 0; i--){
		printf("  %llu",BitVal(mask_TOL_reference,i));
	}
	printf("\n");
}


void printAnglesParallel(int ParallelPoints, int nbParameters, simplex_t *inputSimplex){
	//
	for(int i=0; i<ParallelPoints; i++){
		//
		printVertex(nbParameters, &((inputSimplex + i)->vertex[0]) );
	}
}

//double calculateCONDsimplex
