#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May 20 12:34:21 2019

@author: leonardo
"""

import glob
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import os
import pandas as pd
from scipy.linalg import norm
import time

#WidowX robotic arm base high in cm
hbase=9
#WidowX robotic arm base radio in cm
rbase=11

#Cilinders radio in cm
R=2
        
def listFileNameData(columnName):
    """This function lists the file names in the project folder"""   
    #a = {}
    index = 0
    for filename in glob.glob('*.csv'):
        print(os.path.splitext(filename)[0])
        #key = os.path.splitext(filename)[0]
        #a[key] = index
        fileData = pd.read_csv(filename, skipinitialspace=True, header=None)
        #a[key] = fileData[columnName]
        index += 1
    return fileData

#Function to draw a circle    
def pintacirculo(p0,p1,r):
    v = p1 - p0
    mag = norm(v)
    v = v / mag
    not_v = np.array([1, 0, 0])
    
    if (v == not_v).all():
        not_v = np.array([0, 1, 0])

    n1 = np.cross(v, not_v)
    n1 /= norm(n1)
    n2 = np.cross(v, n1)
    t = np.linspace(0, mag, 2)
    theta = np.linspace(0, 2 * np.pi, 100)
    rsample = np.linspace(0, R, 2)
    t, theta2 = np.meshgrid(t, theta)
    rsample,theta = np.meshgrid(rsample, theta)
    X, Y, Z = [p0[i] + v[i] * t + R * np.sin(theta2) * n1[i] + R * np.cos(theta2) *       n2[i] for i in [0, 1, 2]]
    X2, Y2, Z2 = [p0[i] + rsample[i] * np.sin(theta) * n1[i] + rsample[i] * np.cos(theta) * n2[i] for i in [0, 1, 2]]
    X3, Y3, Z3 = [p0[i] + v[i]*mag + rsample[i] * np.sin(theta) * n1[i] + rsample[i] * np.cos(theta) * n2[i] for i in [0, 1, 2]]
    
    return  X, Y, Z,X2, Y2, Z2, X3, Y3, Z3  
   
#Forward kinematics equations
def FK_04(theta):
    
    px_04 = -0.5e1 * np.cos(theta[0]) * (-0.3e1 * np.cos(-theta[4]) * np.sin(-theta[1]+1.5708) * np.cos(-theta[3]) + 0.3e1 * np.cos(-theta[4]) * np.cos(-theta[1]+1.5708) * np.sin(-theta[3]) + 0.3e1 * np.sin(-theta[4]) * np.sin(-theta[1]+1.5708) * np.sin(-theta[3]) + 0.3e1 * np.sin(-theta[4]) * np.cos(-theta[1]+1.5708) * np.cos(-theta[3]) - 0.3e1 * np.sin(-theta[1]+1.5708) * np.cos(-theta[3]) + 0.3e1 * np.cos(-theta[1]+1.5708) * np.sin(-theta[3]) - np.sin(-theta[1]+1.5708) - 0.3e1 * np.cos(-theta[1]+1.5708))
    py_04 = -0.5e1 * np.sin(theta[0]) * (-0.3e1 * np.cos(-theta[4]) * np.sin(-theta[1]+1.5708) * np.cos(-theta[3]) + 0.3e1 * np.cos(-theta[4]) * np.cos(-theta[1]+1.5708) * np.sin(-theta[3]) + 0.3e1 * np.sin(-theta[4]) * np.sin(-theta[1]+1.5708) * np.sin(-theta[3]) + 0.3e1 * np.sin(-theta[4]) * np.cos(-theta[1]+1.5708) * np.cos(-theta[3]) - 0.3e1 * np.sin(-theta[1]+1.5708) * np.cos(-theta[3]) + 0.3e1 * np.cos(-theta[1]+1.5708) * np.sin(-theta[3]) - np.sin(-theta[1]+1.5708) - 0.3e1 * np.cos(-theta[1]+1.5708))
    pz_04 = -0.15e2 * np.cos(-theta[4]) * np.sin(-theta[1]+1.5708) * np.sin(-theta[3]) - 0.15e2 * np.cos(-theta[4]) * np.cos(-theta[1]+1.5708) * np.cos(-theta[3]) + 0.15e2 * np.sin(-theta[4]) * np.cos(-theta[1]+1.5708) * np.sin(-theta[3]) - 0.15e2 * np.sin(-theta[4]) * np.sin(-theta[1]+1.5708) * np.cos(-theta[3]) - 0.15e2 * np.cos(-theta[1]+1.5708) * np.cos(-theta[3]) - 0.15e2 * np.sin(-theta[1]+1.5708) * np.sin(-theta[3]) - 0.5e1 * np.cos(-theta[1]+1.5708) + 0.15e2 * np.sin(-theta[1]+1.5708)
    
    px_03 = -0.5e1 * np.cos(theta[0]) * (-0.3e1 * np.sin(-theta[1]+1.5708) * np.cos(-theta[3]) + 0.3e1 * np.cos(-theta[1]+1.5708) * np.sin(-theta[3]) - np.sin(-theta[1]+1.5708) - 0.3e1 * np.cos(-theta[1]+1.5708))
    py_03 = -0.5e1 * np.sin(theta[0]) * (-0.3e1 * np.sin(-theta[1]+1.5708) * np.cos(-theta[3]) + 0.3e1 * np.cos(-theta[1]+1.5708) * np.sin(-theta[3]) - np.sin(-theta[1]+1.5708) - 0.3e1 * np.cos(-theta[1]+1.5708))
    pz_03 = -0.15e2 * np.cos(-theta[1]+1.5708) * np.cos(-theta[3]) - 0.15e2 * np.sin(-theta[1]+1.5708) * np.sin(-theta[3]) - 0.5e1 * np.cos(-theta[1]+1.5708) + 0.15e2 * np.sin(-theta[1]+1.5708)
    
    px_02 = 0.5e1 * np.cos(theta[0]) * (np.sin(-theta[1]+1.5708) + 0.3e1 * np.cos(-theta[1]+1.5708))
    py_02 = 0.5e1 * np.sin(theta[0]) * (np.sin(-theta[1]+1.5708) + 0.3e1 * np.cos(-theta[1]+1.5708))
    pz_02 = -0.5e1 * np.cos(-theta[1]+1.5708) + 0.15e2 * np.sin(-theta[1]+1.5708)
    
    px_01 = 0.15e2 * np.cos(theta[0]) * np.cos(-theta[1]+1.5708)
    py_01 = 0.15e2 * np.sin(theta[0]) * np.cos(-theta[1]+1.5708)
    pz_01 = 0.15e2 * np.sin(-theta[1]+1.5708)
    
    return [px_01, px_02, px_03, px_04, py_01, py_02, py_03, py_04, pz_01, pz_02, pz_03, pz_04]
     

#Function to print WidowX trajectory calculated with the ZedBoard    
def print_arm(buff):
    p0 = np.array([0,0,0])
    p1 = np.array([0,0,hbase])
    X, Y, Z,X2, Y2, Z2, X3, Y3, Z3=pintacirculo(p0,p1,rbase)
    
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    
    ax.set_xlim3d(-40, 40)
    ax.set_ylim3d(-40, 40)
    ax.set_zlim3d(0, 50)
    
    ax.plot_surface(X, Y, Z, color='blue')
    ax.plot_surface(X2, Y2, Z2, color='blue')
    ax.plot_surface(X3, Y3, Z3, color='blue')
    
    if len(buff)>0:
        for i in range(len(buff)-1):
            fk=FK_04(buff[i])
            x = np.array([0, fk[0], fk[1], fk[2], fk[3]])
            y = np.array([0, fk[4], fk[5], fk[6], fk[7]])
            z = np.array([hbase, fk[8], fk[9], fk[10], fk[11]])
            #print(str(i) + "--> new position [ "+ str(x[4]) +", "+ str(y[4]) +", "+ str(z[4]) + "]")
            print("new position [ "+ str(x[4]) +", "+ str(y[4]) +", "+ str(z[4]) + "]")
            #print(str(i) + "--> new position [ "+ str(x[4]) + "]")
            a=np.array([x,y,z])
            line=ax.plot(a[0],a[1],a[2])
            plt.setp(line, linestyle='-',linewidth=4.0)
            #plt.waitforbuttonpress()
            #time.sleep(.300)
            plt.pause(0.05)
            if i<(len(buff)-1):
                ax.lines.pop(0)
            plt.draw()
            
            
""" Here the main of the function to analyze the measurement data""" 
       

if __name__ == "__main__": 
    print("...starting")
    a = listFileNameData("ciao")
    buff = []
    #buff = [[0 for x in range(a.shape[1])] for y in range(a.shape[0])]
    for lineBuffer in range(a.shape[0]):
        #print(lineBuffer)
        buff.append(a.loc[lineBuffer,:].values.tolist())
    print_arm(buff)
